package menu

import (
	model2 "ArmedPolice/app/common/model"
	"ArmedPolice/app/controller"
	"ArmedPolice/app/model"
	"ArmedPolice/app/service"
	"ArmedPolice/serve/orm"
	"errors"

	"gorm.io/gorm"
)

type Role struct{ *controller.Platform }

type RoleHandle func(session *service.Session) *Role

// List 列表信息
func (c *Role) List(roleID uint64) ([]*TreeChecked, error) {
	mSysMenu := model.NewSysMenu()

	return menuForRole(mSysMenu, c.TenantID, roleID)
}

// Bind 绑定菜单
func (c *Role) Bind(roleID uint64, menuIDs []uint64) error {
	if len(menuIDs) > 0 {
		var count int64

		mSysMenu := model.NewSysMenu()

		if err := model2.Count(mSysMenu.SysMenu, &count, model2.NewWhereIn("id", menuIDs),
			model2.NewWhere("auth", model2.SysMenuAuthForSystem)); err != nil {
			return err
		} else if count > 0 {
			return errors.New("不可设置超管菜单")
		}
	}
	mSysRoleMenu := model.NewSysRoleMenu()

	return orm.GetDB().Transaction(func(tx *gorm.DB) error {
		err := model2.DeleteWhere(mSysRoleMenu.SysRoleMenu, []*model2.ModelWhere{model2.NewWhere("role_id", roleID)}, tx)

		if err != nil {
			return err
		}
		if len(menuIDs) <= 0 {
			return nil
		}
		menus := make([]*model2.SysRoleMenu, 0)

		mark := make(map[uint64]uint64, 0)

		for _, v := range menuIDs {
			if _, has := mark[v]; has {
				continue
			}
			menus = append(menus, &model2.SysRoleMenu{
				ModelTenant: model2.ModelTenant{TenantID: c.TenantID}, RoleID: roleID, MenuID: v,
			})
			mark[v] = v
		}
		if err = model2.Creates(mSysRoleMenu.SysRoleMenu, menus, tx); err != nil {
			return err
		}
		return nil
	})
}

func NewRole() RoleHandle {
	return func(session *service.Session) *Role {
		return &Role{Platform: &controller.Platform{Session: session}}
	}
}
