package menu

import (
	model2 "ArmedPolice/app/common/model"
	"ArmedPolice/app/controller"
	"ArmedPolice/app/model"
	"ArmedPolice/app/service"
	"ArmedPolice/serve/orm"
	"errors"

	"gorm.io/gorm"
)

type Tenant struct{ *controller.Platform }

type TenantHandle func(session *service.Session) *Tenant

func (c *Tenant) auth() {

}

// List 列表信息
func (c *Tenant) List(tenantID uint64) ([]*TreeChecked, error) {
	mSysMenu := model.NewSysMenu()
	return menuForTenant(mSysMenu, tenantID)
}

// Bind 绑定菜单
func (c *Tenant) Bind(tenantID uint64, menuIDs []uint64) error {
	if len(menuIDs) > 0 {
		var count int64

		mSysMenu := model.NewSysMenu()

		if err := model2.Count(mSysMenu.SysMenu, &count, model2.NewWhereIn("id", menuIDs),
			model2.NewWhere("auth", model2.SysMenuAuthForSystem)); err != nil {
			return err
		} else if count > 0 {
			return errors.New("不可设置超管菜单")
		}
	}
	mSysTenantMenu := model.NewSysTenantMenu()

	return orm.GetDB().Transaction(func(tx *gorm.DB) error {
		// TODO：先全部删除，后期考虑局部删除
		err := model2.DeleteWhere(mSysTenantMenu.SysTenantMenu, []*model2.ModelWhere{model2.NewWhere("tenant_id", tenantID)}, tx)

		if err != nil {
			return err
		}
		// 租户角色的菜单
		mSysRoleMenu := model.NewSysRoleMenu()

		if len(menuIDs) <= 0 {
			if err = model2.DeleteWhere(mSysRoleMenu.SysRoleMenu, []*model2.ModelWhere{model2.NewWhere("tenant_id", tenantID)}, tx); err != nil {
				return err
			}
			return nil
		}
		menus := make([]*model2.SysTenantMenu, 0)

		mark := make(map[uint64]uint64, 0)

		for _, v := range menuIDs {
			if _, has := mark[v]; has {
				continue
			}
			menus = append(menus, &model2.SysTenantMenu{
				ModelTenant: model2.ModelTenant{TenantID: tenantID}, MenuID: v,
			})
			mark[v] = v
		}
		if err = model2.Creates(mSysTenantMenu.SysTenantMenu, menus, tx); err != nil {
			return err
		}
		// 删除租户下角色不存在的菜单
		if err = model2.DeleteWhere(mSysRoleMenu.SysRoleMenu, []*model2.ModelWhere{
			model2.NewWhere("tenant_id", tenantID),
			model2.NewWhereNotIn("menu_id", menuIDs),
		}); err != nil {
			return err
		}
		return nil
	})
}

func NewTenant() TenantHandle {
	return func(session *service.Session) *Tenant {
		return &Tenant{Platform: &controller.Platform{Session: session}}
	}
}
