package model

import (
	"TenantServe/app/common/model"
	"TenantServe/serve/orm"
	"fmt"
)

type SysUserTenant struct {
	*model.SysUserTenant
}

type (
	// SysUserTenantBasic 基本信息
	SysUserTenantBasic struct {
		ID     uint64                    `json:"id"`
		Name   string                    `json:"name"`
		Mobile string                    `json:"mobile"`
		Status model.SysUserTenantStatus `json:"status"`
	}
	// SysUserTenantUser 用户信息
	SysUserTenantUser struct {
		*SysUserTenantBasic
		UID      uint64                      `json:"uid"`
		Email    string                      `json:"email"`
		Identity model.SysUserTenantIdentity `json:"identity"`
		Remark   string                      `json:"remark"`
	}
)

// Member 成员信息
func (m *SysUserTenant) Member(page, pageSize int, count *int64, where ...*model.ModelWhere) ([]*SysUserTenantBasic, error) {
	mSysUser := NewSysUser()

	db := orm.GetDB().Table(m.TableName()+" AS u_t").
		Select("u_t.id, u.name, u.mobile, u_t.status").
		Joins(fmt.Sprintf("LEFT JOIN %s AS u ON u_t.uid = u.uuid", mSysUser.TableName())).
		Where("u_t.is_deleted = ? AND u.is_deleted = ?", model.DeleteStatusForNot, model.DeleteStatusForNot)

	if len(where) > 0 {
		for _, wo := range where {
			db = db.Where(wo.Condition, wo.Value)
		}
	}
	out := make([]*SysUserTenantBasic, 0)

	if err := db.Count(count).Error; err != nil {
		return nil, err
	}
	if err := db.Order("u_t.id " + model.OrderModeToDesc).Offset((page - 1) * pageSize).Limit(pageSize).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// User 用户信息
func (m *SysUserTenant) User(page, pageSize int, count *int64, where ...*model.ModelWhere) ([]*SysUserTenantUser, error) {
	mSysUser := NewSysUser()

	db := orm.GetDB().Table(m.TableName()+" AS u_t").
		Select("u_t.id, u_t.uid, u.name, u.mobile, u.email, u_t.identity, u_t.status, u.remark, u_t.created_at").
		Joins(fmt.Sprintf("LEFT JOIN %s AS u ON u_t.uid = u.uuid", mSysUser.TableName())).
		Where("u_t.is_deleted = ? AND u.is_deleted = ?", model.DeleteStatusForNot, model.DeleteStatusForNot)

	if len(where) > 0 {
		for _, wo := range where {
			db = db.Where(wo.Condition, wo.Value)
		}
	}
	out := make([]*SysUserTenantUser, 0)

	if err := db.Count(count).Error; err != nil {
		return nil, err
	}
	if err := db.Order("u_t.id " + model.OrderModeToDesc).Offset((page - 1) * pageSize).Limit(pageSize).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

func NewSysUserTenant() *SysUserTenant {
	return &SysUserTenant{SysUserTenant: model.NewSysUserTenant()}
}
